(*******************************************************************************

       Delphi Runtime Library
       System Unit

       Changes by NicoDE

       No memory manager, no try-finally, no try-except, no compiler-magic
       concerning string<>pchar conversion (casting).
       Just initialisation of EXE and most basic code. These units should be
       compatible with Delphi 4 & 5 and require no changes of the Windows.pas.
 *******************************************************************************)
unit System;

interface

procedure _InitExe;
procedure _HandleFinally;
procedure _halt0;

type
  PGUID = ^TGUID;
  TGUID = record
    D1: LongWord;
    D2: Word;
    D3: Word;
    D4: array[0..7] of Byte;
  end;

{$IFDEF VER120}
  HRESULT = type LongWord; { Delphi 4 }
{$ELSE}
  HRESULT = type LongInt; { Delphi 5 }
{$ENDIF}
  PShortString = ^ShortString;
  PAnsiString = ^AnsiString;
  PWideString = ^WideString;
  PString = PAnsiString;
  PExtended = ^Extended;
  PCurrency = ^Currency;
  PVariant = ^Variant;
  POleVariant = ^OleVariant;
  PInt64 = ^Int64;
  TDateTime = type Double;
  PDateTime = ^TDateTime;

  PLongWord = ^LongWord;
  PWord = ^Word;

const
  Kernel32 = 'kernel32.dll';
  User32 = 'user32.dll';

var
  HKernel32: LongWord;
  HUser32: LongWord;

type
  TFNExitProcess = procedure(uExitCode: LongWord); stdcall;

var
  ExitCode: LongWord;
  ExitProcess: TFNExitProcess;

procedure Move(const Source; var Dest; Count: Integer);
procedure _FillChar;
function LoadLibraryA(lpLibFileName: PAnsiChar): LongWord; stdcall;
function LoadLibraryW(lpLibFileName: PWideChar): LongWord; stdcall;
function LoadLibrary(lpLibFileName: PChar): LongWord; stdcall;
function GetProcAddress(hModule: LongWord; lpProcName: PChar): Pointer; stdcall;

implementation

function LoadLibraryA; external kernel32 name 'LoadLibraryA';
function LoadLibraryW; external kernel32 name 'LoadLibraryW';
function LoadLibrary; external kernel32 name 'LoadLibraryA';
function GetProcAddress; external kernel32 name 'GetProcAddress';

procedure Move(const Source; var Dest; count: Integer);
asm
  PUSH    ESI
  PUSH    EDI
  MOV     ESI,EAX
  MOV     EDI,EDX
  MOV     EAX,ECX
  CMP     EDI,ESI
  JA      @@down
  JE      @@exit
  SAR     ECX,2
  JS      @@exit
  REP     MOVSD
  MOV     ECX,EAX
  AND     ECX,03H
  REP     MOVSB
  JMP     @@exit
@@down:
  LEA     ESI,[ESI+ECX-4]
  LEA     EDI,[EDI+ECX-4]
  SAR     ECX,2
  JS      @@exit
  STD
  REP     MOVSD
  MOV     ECX,EAX
  AND     ECX,03H
  ADD     ESI,4-1
  ADD     EDI,4-1
  REP     MOVSB
  CLD
@@exit:
  POP     EDI
  POP     ESI
end;

procedure _FillChar;
asm
  PUSH    EDI
  MOV     EDI,EAX
  MOV     CH,CL
  MOV     EAX,ECX
  SHL     EAX,16
  MOV     AX,CX
  MOV     ECX,EDX
  SAR     ECX,2
  JS      @@exit
  REP     STOSD
  MOV     ECX,EDX
  AND     ECX,3
  REP     STOSB
@@exit:
  POP     EDI
end;

procedure _InitExe;
const
  PExitProcess: PChar = 'ExitProcess';
  PKernelModul: PChar = Kernel32;
  PUserModul: PChar = User32;
asm
  PUSH PExitProcess
  PUSH PUserModul
  CALL LoadLibrary
  MOV  HUser32, EAX
  PUSH PKernelModul
  CALL LoadLibrary
  MOV  HKernel32, EAX
  PUSH EAX
  CALL GetProcAddress
  MOV  ExitProcess, EAX
end;

procedure _HandleFinally;
asm
   // dummy
end;

procedure _halt0;
asm
  PUSH ExitCode
  CALL ExitProcess
end;

end.

